from stonks.stonks import run_stonks_analysis
from stonks.version import version

__version__ = version.version
__author__ = "Daniel Lasota"
__description__ = "Script that scrapes market Data from Yahoo Finance Api and performs base analysis with plotting"
__email__ = "grossmann.root@gmail.com"
__url__ = "https://www.youtube.com/watch?v=dQw4w9WgXcQ"
